{% extends 'base.html' %}


{% block content %}
<div class="py-4 px-5">
    [# if list_path_name #]
    <a href="{% url "[[ list_path_name ]]" %}" class="btn btn-light"> < back</a>
    [# endif #]
    <h3 class="mt-3">[[ item ]] detail</h3>
    <div class="card card-body w-50">
        <dl class="row">
            [# for field in fields #]
                <dt class="col-sm-2">[[ field.verbose_name ]]</dt>
                <dd class="col-sm-10">{{ [[ item ]].[[ field.name ]] }}</dd>
            [# endfor #]
        </dl>
        <div>
        [# if update_path_name #]
            <a href="{% url "[[ update_path_name ]]" [[ item ]].pk %}" class="btn btn-dark">edit</a>
        [# endif #]
        </div>
    </div>
</div>
{% endblock %}
