{% extends 'base.html' %}

{% load crispy_forms_tags %}


{% block content %}
<div class="d-flex w-100 justify-content-center py-4">
    <div class="card card-body shadow-sm" style="max-width: 500px">
        <form method="post">
            <h3 class="fw-bold">Update [[ model.lower() ]]</h3>
            {% csrf_token %}
            {{ form|crispy }}

            <button class="btn btn-dark">submit</button>
        </form>
    </div>
</div>
{% endblock %}